//XXXXXXXXXXXXXXXX HCS12(X) SPI Master software example XXXXXXXXXXXXXXXXXXXXXXXXX
/*
********************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2010 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO A PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods
supplied under this Agreement.
******************************************************************************
Project Name:  main.c                                                      
Last modified: feb/24/2011
By:            B34874                                                      
******************************************************************************
Description: MC9S12(X) - SCI default utility
This software is classified as Engineering Sample Software.                
******************************************************************************
this sample software was tested on: MC9S12XEP100/EVB9S12XEP100 Rev.B 
the sample software does:
*/

#include <hidef.h>      /* common defines and macros */
#include <MC9S12XEP100.h>     /* derivative information */
#pragma LINK_INFO DERIVATIVE "mc9s12xep100"

#define SIZE_OF_PACKETrx  5       //size of receiver buffer - constant or you can use variable size of packet
#define SIZE_OF_PACKETtx  10      //size of receiver buffer - constant or you can use variable size of packet

volatile unsigned char packet_lenght;
volatile unsigned char sc0_data_in;
volatile unsigned char buffer[SIZE_OF_PACKETrx];

volatile unsigned char pointer_to_buffer = 0;

volatile unsigned char buffer_full = 0;              //flag - full buffer

volatile unsigned char *ptr_g;                       //global pointer to data

volatile unsigned char i,ninth_bit;

unsigned char packet[SIZE_OF_PACKETtx];


//==============================================================================
//SCI block initialization
//==============================================================================
void SC0_Init(void)   
{
   //SCI Baud Rate SBR[12:0] = bus clock / (16*Baud rate) 
   //or SBR[12:1] = bus clock / (32*Baud rate) for case of IREN=1(Infrared)
   //in this case bus clock = 8MHz, so SBR[12:0] = 8000000 / (16*9600) = 52 = 0x0034
                     //------------------------------------------------------
   SCI0BDH = 0x00;    //  Infrared options, higher 5bits of prescaler
                     /*  0b00000000
                      *    ||||||||__ SBR8 SCI Baud Rate Bits
                      *    |||||||___ SBR9 
                      *    ||||||____ SBR10
                      *    |||||_____ SBR11
                      *    ||||______ SBR12 
                      *    |||_______ TNP0 Transmitter Narrow Pulse Bits /00-11=Narrow Pulse Width 3/16,1/16,1/32,1/4
                      *    ||________ TNP1 
                      *    |_________ IREN Infrared Enable Bit /1=Infrared
                      */
                     //------------------------------------------------------
   SCI0BDL = 0x34;    // prescaller = 0x34   9600Bd  (8MHz bus clock is set)
                     /*  0b00110100
                      *    ||||||||__ SBR0 SCI Baud Rate Bits
                      *    |||||||___ SBR1 
                      *    ||||||____ SBR2
                      *    |||||_____ SBR3
                      *    ||||______ SBR4 
                      *    |||_______ SBR5
                      *    ||________ SBR6 
                      *    |_________ SBR7
                      */
                     //------------------------------------------------------
   SCI0CR1 = 0x84;    // SCI Control Register 1       
                     /*  0b10000100
                      *    ||||||||__ PT     Parity Type Bit /1=Odd parity,0=Even parity
                      *    |||||||___ PE     Parity Enable Bit /1=Parity function enabled
                      *    ||||||____ ILT    Idle Line Type Bit /Idle character bit count begins after (0= start)1=stop bit
                      *    |||||_____ WAKE   Wakeup Condition Bit /1=Address mark wakeup,0=Idle line wakeup
                      *    ||||______ M      Data Format Mode Bit /One start bit (0=eight data bits)1=nine data bits, one stop bit
                      *    |||_______ RSRC   Receiver Source Bit /1=Receiver input connected externally to transmitter
                      *    ||________ SCIWAI SCI Stop in Wait Mode Bit /1=SCI disabled in wait mode
                      *    |_________ LOOPS  Loop Select Bit /1=Loop operation enabled
                      */
                     // LOOPS=0 Normal operation
                     // LOOPS=1,RSRC=0 Loop mode with Rx input internally connected to Tx output 
                     // LOOPS=RSRC=1 Single-wire mode with Rx input internally connected to TXD pin, RXD pin disconnect
                     //------------------------------------------------------                      
   SCI0CR2 = 0x2C;    // SCI Control Register 2       
                     /*  0b00101100
                      *    ||||||||__ SBK  Send Break Bit /1=Transmit break characters
                      *    |||||||___ RWU  Receiver Wakeup Bit /1=enables the wakeup function receiver interrupt requests.
                      *    ||||||____ RE   Receiver Enable Bit /1=Receiver enabled
                      *    |||||_____ TE   Transmitter Enable Bit /1=Transmitter enabled
                      *    ||||______ ILIE Idle Line Interrupt Enable Bit /1=IDLE interrupt requests enabled
                      *    |||_______ RIE  Receiver Full Interrupt Enable Bit /1=RDRF and OR interrupt requests enabled
                      *    ||________ TCIE Transmission Complete Interrupt Enable Bit /1=TC interrupt requests enabled
                      *    |_________ TIE  Transmitter Interrupt Enable Bit /1=TDRE interrupt requests enabled
                      */
                     // RDRF=receive data register full flag                             
                     // OR=overrun flag                          
                     // TC=transmission complete flag
                     // TDRE=transmit data register empty flag                                                          
                     //------------------------------------------------------
                    
}


//==============================================================================
//SCI interrupt runtine
//==============================================================================
#pragma CODE_SEG NON_BANKED
interrupt 20 void SCI0_Isr(void)  
{ 
//---------------
//prection against randomly flag clearing
 unsigned char scicr2,scisr1;
 scisr1 = SCI0SR1;                          // save status register actual status
 scicr2 = SCI0CR2;                          // save control register actual status
//---------------

//******************************************************************************
//Interrupt runtine RECEIVER part
//******************************************************************************

//if receiver interrupt is enabled and corresponding interrupt flag is set
 if((scicr2 & SCI0CR2_RIE_MASK) && ((scisr1 & (SCI0SR1_OR_MASK | SCI0SR1_RDRF_MASK))))
  {
    if(scisr1 & SCI0SR1_OR_MASK)            // if overrun error do nothing/something
      {
        (void)SCI0DRL;                            // clear interrupt flag 
        // do something
      }
    else
      {
        //ninth_bit=SCI0DRH_R8;              //R8 is the ninth data bit received when the SCI is configured for 9-bit data format (M = 1).
        sc0_data_in = SCI0DRL;             // read received character + clear interrupt flag
        //---------------
        //receive packet and store to buffer
        buffer[pointer_to_buffer] = sc0_data_in;
        pointer_to_buffer++;
        if (pointer_to_buffer >= SIZE_OF_PACKETrx)
        {
            pointer_to_buffer = 0;
            
            //do something with data in buffer
            //or set flag and do something in main program
            //buffer_full = 1;  //flag for main program
        }
        
        /*
        //---------------
        //receive packet and store to buffer with waiting to packet begin ('#' as start character)
        if(((pointer_to_buffer == 0) || (pointer_to_buffer == 1)) && (sc0_data_in != '#'))
        {
          pointer_to_buffer = 0;  //we wait for packet which has to start with ##          
        } 
        else
        {          
          buffer[pointer_to_buffer] = sc0_data_in;
          pointer_to_buffer++;
          if (pointer_to_buffer >= SIZE_OF_PACKETrx)
          {
            pointer_to_buffer = 0;
            
            //do something with data in buffer            
            //or set flag and do something in main program
            //buffer_full = 1;  //flag for main program
          }
        }
        */
        
        
      }
  }
  

//******************************************************************************
//Interrupt runtine TRANCEIVER part
//******************************************************************************

  //if tranceiver interrupt is enabled and corresponding interrupt flag is set
  if((scicr2 & SCI0CR2_TIE_MASK) && ((scisr1 & SCI0SR1_TDRE_MASK)))
  { 
    while(!SCI0SR1_TDRE);                       //wait for transmit data register empty flag
    SCI0DRL = *ptr_g;                           //Send data
    *ptr_g++;
    packet_lenght--;
    
    if (packet_lenght == 0)                     //end of packet detection
          {
            SCI0CR2_TIE = 0;                    //TDRE interrupt requests disabled
          }
    
  }
  
}
#pragma CODE_SEG DEFAULT

//==============================================================================
//SCI send variable
//==============================================================================
//this function will be active until all data will be send
void SCI0_Send_Variable(unsigned char *ptr, unsigned char length) 
{
  unsigned char i;
  
  for(i=0; i<length; i++)
  {
    while(!SCI0SR1_TDRE);            //wait for transmit data register empty flag
    SCI0DRL = *ptr++;  
  }  
}

//==============================================================================
//SCI send Data    +    ninth data bit when the SCI is configured for 9-bit data format (M = 1).
//==============================================================================
//this function will be active until all data will be send
void SCI0_Send_Data(unsigned char data, unsigned char T8) 
{
  while(!SCI0SR1_TDRE);             //wait for transmit data register empty flag
  SCI0DRH_T8=T8;                    //T8 is the ninth data bit transmitted when the SCI is configured for 9-bit data format (M = 1)
  SCI0DRL = data;  
    
}

//==============================================================================
//send packet trough SCI with interrupt control
//==============================================================================
//this function will set up data buffer and leave sending agenda to interrupt runtine
void SCI0_Send_Packet(unsigned char *ptr, unsigned char length) 
{
  ptr_g = ptr;
  packet_lenght = length;
  SCI0CR2_TIE = 1;                  //TDRE interrupt requests enabled
}

//==============================================================================
//main
//==============================================================================
void main(void) {
  
  SC0_Init();
  
  EnableInterrupts;
  
  
  //fill the packet
  for(i=0;i<10;i++)
  {
    packet[i] = (unsigned char)i + 48; //increment number line, +48 presents matching to ASCII
  }
  
  //send packet
  SCI0_Send_Variable(&(unsigned char)packet[0], sizeof(packet));
  //send ASCII underscore 0x5F = 95 = '_'
  SCI0_Send_Data(0x5F,0);   
  //send packet with interrupt control 
  SCI0_Send_Packet(&(unsigned char)packet[0], sizeof(packet));
  //already sended ASCII text:0123456789_0123456789 

  /* put your own code here */

  for(;;) {} /* wait forever */
  /* please make sure that you never leave this function */
}
